/*
 * arch/i86/drivers/char/tty.c - A simple tty driver
 * (C) 1997 Chad Page et. al
 */

/* 
 * This new tty sub-system builds around the character queue to provide a
 * VFS interface to the character drivers (what a mouthful! :)  
 */

#include <linuxmt/types.h>
#include <linuxmt/config.h>
#include <linuxmt/sched.h>
#include <linuxmt/fs.h>
#include <linuxmt/fcntl.h>
#include <linuxmt/errno.h>
#include <linuxmt/termios.h>
#include <linuxmt/chqueue.h>
#include <linuxmt/ntty.h>
#include <linuxmt/major.h>

struct tty ttys[8];
extern struct tty_ops dircon_ops;

/* Just open it for now :) */
int tty_open(inode,file)
struct inode *inode;
struct file *file;
{
	return Console_open(inode, file);
}

void tty_release(inode,file)
struct inode *inode;
struct file *file;
{
	int minor=MINOR(inode->i_rdev);
	struct tty *tty=file->private_data;
}

/*
 *	Write to a terminal
 */
 
int tty_write(inode,file,data,len)
struct inode *inode;
struct file *file;
char *data;
int len;
{
	int minor=MINOR(inode->i_rdev);
	struct tty *tty=file->private_data;

	int i;

	for (i = 0; i < len; i++)
		con_charout(peekb(current->t_regs.ds, data + i)); 

	return i;
}

int tty_read(inode,file,data,len)
struct inode *inode;
struct file *file;
char *data;
int len;
{
	int i = 0, j = 0;
	unsigned char ch;

	while (((i < len) && (j != 10))) { 
		j = chq_getch(&ttys[0].inq, &ch, 1);
		if ((j != -1) && (j != '\b')) {
			pokeb(current->t_regs.ds, (data + i++), ch);		
			con_charout(ch);
		}
		if (((j == '\b') && (i > 0))) {
			i--;	
			con_charout(ch);
		}	
	};	
	return i;
}

int tty_ioctl(inode,file, cmd, arg)
struct inode *inode;
struct file *file;
int cmd;
char *arg;
{
	return -EINVAL;
}

int tty_lseek(inode,file,offset,origin)
struct inode *inode;
struct file *file;
off_t offset;
int origin;
{
	return -ESPIPE;
}

int tty_readdir()
{
	return -ENOTDIR;
}

static struct file_operations tty_fops =
{
	tty_lseek,
	tty_read,
	tty_write,
	tty_readdir,
	NULL,		/* Select - needs doing */
	tty_ioctl,
	tty_open,
	tty_release,
	NULL,
	NULL,
	NULL
};

void tty_init()
{
	int i;

	for (i = 0; i < NUM_TTYS; i++) { 
		chq_init(&ttys[i].inq, ttys[i].inq_buf, INQ_SIZE);
		chq_init(&ttys[i].outq, ttys[i].outq_buf, OUTQ_SIZE);
	}

	ttys[0].ops = &dircon_ops;
	register_chrdev(TTY_MAJOR,"tty",&tty_fops);
}

